from .PyTriliumCustomClient import PyTriliumCustomClient

class PyTrilium(PyTriliumCustomClient):
    def __init__(self, url, token=None, password=None, debug=False) -> None:
        """Initializes the PyTrilium class. You need to either provide an ETAPI token OR a password (which will then be used to generate an ETAPI token).

        Parameters
        ----------
        url : str
            The URL of the Trilium instance. This should include the protocol (http:// or https://) and the port if it is not the protocol's respective port (443 for https, 80 for http). e.g. `https://trilium.example.com:8080`
        token : str, optional
            The token for the Trilium instance. This can be found in the Trilium settings.
        debug : bool, optional
            If you would like to enable debugging, set this to True, by default False
        password : str, optional
            The password for the Trilium instance. This can be found in the Trilium settings. This is only required if you are using Trilium's built-in authentication, by default None
        """
        super().__init__(url, token, debug)

        # Set up the requests session, the validate that either a password or a token was provided
        # If not, return an error
        self.make_requests_session()
        if not token and not password:
            raise ValueError("You must provide either a token or a password.")
        if password:
            self.token = self.auth_login(password)
        if token:
            self.token = token
        self.set_session_auth(self.token)

        # Attempt a basic call to make sure that the token is valid
        self.attempt_basic_call()

    def auth_login(self, password:str) -> str:
        """Authenticate to Trilium using a password. This should not be called manually. This will return the token that can be used to authenticate to Trilium in future requests.

        Parameters
        ----------
        password : str
            The password to send to Trilium

        Returns
        -------
        str
            The ETAPI token that can be used to authenticate to Trilium in future requests.
        """

        # Returns {"authToken": "33xHpBRHetAN_fh3g0B7MQaaaaj1871fbXLbbK4JAT06GGmZOSwet56M="}
        data = {"password": password}

        resp = self.make_request("/auth/login", data=data, method="POST")
        return resp.json()["authToken"]
    
    def auth_logout(self):
        """Logs out of Trilium. This should not be called manually."""
        self.make_request("/auth/logout", method="POST")

    def get_inbox_note(self, date:str):
        """Get the inbox's note for a date.

        Parameters
        ----------
        date : str
            The date which you would like to fetch the inbox note for. This should be in the format of YYYY-MM-DD. e.g. `2021-01-01`
        """
        self.make_request(f"/inbox/{date}")
    
    def print_custom_functions(self):

        dont_show_these_funcs = ["url", "token", "session"]

        list_of_funcs = dir(self)
        for func in list_of_funcs:
            if "__" not in func and "logger" not in func:
                if func not in dont_show_these_funcs:
                    print(func)


import requests
from .PyTriliumClient import PyTriliumClient


class PyTriliumAttributeClient(PyTriliumClient):
    def __init__(self, url, token, debug=False) -> None:
        super().__init__(url, token, debug)

    def get_attribute_by_id(self, attribute_id: str) -> dict:
        """Given the Attribute's ID, this will return the Attribute's information.

        Parameters
        ----------
        attribute_id : str
            Trilium's ID for the Attribute, this can be seen by clicking the 'i' on the attribute, near the top.

        Returns
        -------
        dict
            The response from the Trilium API.
        """
        return self.make_request(f"/attributes/{attribute_id}").json()

    def post_attribute(self, data: str) -> dict:
        """This will create a new Attribute.

        Parameters
        ----------
        data : str
            The data to send to the Trilium API.This should be in the format of a string.

        Returns
        -------
        dict
            The JSON response from Trilium, as a dictionary.
        """
        return self.make_request("/attributes", method="POST", data=data).json()



from .PyTriliumNoteClient import PyTriliumNoteClient
from .PyTriliumBranchClient import PyTriliumBranchClient
from .PyTriliumAttributeClient import PyTriliumAttributeClient
from .PyTriliumCalendarClient import PyTriliumCalendarClient

# This class inherits from everything else, but also implements custom functions
# so I'm creating this to help save my sanity in the future
class PyTriliumCustomClient(PyTriliumNoteClient, PyTriliumBranchClient, PyTriliumAttributeClient, PyTriliumCalendarClient):
    def __init__(self, url, token, debug=False) -> None:
        super().__init__(url, token, debug)


import requests
from .PyTriliumClient import PyTriliumClient


class PyTriliumCalendarClient(PyTriliumClient):
    def __init__(self, url, token, debug=False) -> None:
        super().__init__(url, token, debug)

    def get_year_note(self, year: str) -> dict:
        """Get the note for a year, in Trilium's calendar.

        Parameters
        ----------
        year : str
            The year you want to fetch from Trilium's calendar.

        Returns
        -------
        dict
            The JSON response from Trilium, as a dictionary.
        """
        return self.make_request(f"calendar/years/{year}").json()

    def get_weeks_note(self, weeks: str) -> dict:
        """Get the note for a week, in Trilium's calendar.

        Parameters
        ----------
        weeks : str
            The week you want to fetch from Trilium's calendar.

        Returns
        -------
        dict
            The JSON response from Trilium, as a dictionary.
        """
        return self.make_request(f"calendar/weeks/{weeks}").json()
    
    def get_months_note(self, months: str) -> dict:
        """Get the note for a month, in Trilium's calendar.

        Parameters
        ----------
        months : str
            The month you want to fetch from Trilium's calendar.

        Returns
        -------
        dict
            The JSON response from Trilium, as a dictionary.
        """
        return self.make_request(f"calendar/months/{months}").json()
    
    def get_days_note(self, date: str) -> dict:
        """Get the note for a day, in Trilium's calendar.

        Parameters
        ----------
        date : str
            The date you want to fetch from Trilium's calendar.

        Returns
        -------
        dict
            The JSON response from Trilium, as a dictionary.
        """
        return self.make_request(f"calendar/days/{date}").json()